import pygame
from Sprite import *
import config

class MagicRuneGraphic:
	pass

class MagicRune:
	
	def __init__(self, name, mp, recast_time, levelup_uses):
		self.level = 0
		self.name = name
		self.usecFunc = pygame.time.get_ticks
		
		# arrays
		self.mp = mp
		self.recast_time = recast_time
		self.levelup_uses = levelup_uses
				
		self.lastCastTime = self.usecFunc() # reload at start
		
		self.uses = 0
		
		# sprites
		self.pic = Sprite("runes", self.name)
		self.pic.show()
		
		self.level_text = TextSprite(0, 0, str(self.level+1), (255, 255, 255), 20, "burti_bold.ttf")
		
		self.bar = MeterBar(60, 5, 100)
		self.bar.set_amount(100)
		
		self.levelbar = MeterBar(60, 5, 100, False, (255,255,255), (0,0,255))
		self.bar.set_amount(0)
		
		self.setPos( (config.SCREEN_WIDTH/2,10) )
		self.controller = None
	
	def setController(self, ctrl):
		self.controller = ctrl
	
	def update(self, elapsed_time):
		self.bar.set_amount(self.recastStatus() * 100)
		
		if self.getLevel()+1 < len(self.mp):
			self.levelbar.set_amount(float(self.uses) / self.levelup_uses[self.level] * 100)
	
	def draw(self, display):
		self.pic.draw(display)
		self.bar.draw(display)
		self.level_text.draw(display)
		if self.getLevel()+1 < len(self.mp):
			self.levelbar.draw(display)
	
	def setPos(self, pos):
		self.pos = pos
		self.pic.move(*pos)
		self.bar.move(pos[0] + 1, pos[1] + 65)
		self.levelbar.move(pos[0] + 1, pos[1] + 75)
		self.level_text.move(pos[0] + 45, pos[1] + 35)
	
	def move_rel(self, dx, dy):
		self.pic.move_rel(dx, dy)
		self.bar.move_rel(dx, dy)
		self.levelbar.move_rel(dx, dy)
		self.level_text.move_rel(dx, dy)
	
	def increaseUses(self):
		self.uses += 1
		self.lastCastTime = self.usecFunc()
		
		if (self.uses >= self.levelup_uses[self.getLevel()]) and (self.getLevel()+1 < len(self.mp)):
			self.uses = 0
			self.increaseLevel()
		
		
	def increaseLevel(self):
		self.level += 1
		self.level_text.set_text(str(self.level + 1))
		self.controller.addMp(20)
		# additional effects!
	
	def getRequiredMp(self):
		return self.mp[ self.getLevel() ]
	
	def recastStatus(self):
		needed = self.recast_time[ self.getLevel() ]
		return min(1.0, (self.usecFunc() - self.lastCastTime) / float(needed))
	
	def getLevel(self): return self.level

class MagicController:

	def __init__(self, position):
		self.maxmp = 100
		self.mp = self.maxmp
		self.magics = {}
		
		self.recover_time = 500		# usec / 1mp
		self.recoverProgress = 0
		
		self.mpbar = MeterBar(100, 10, 100, False, (0,0,255))
		self.mpbar.move(*(position))
		self.mpbar.set_amount(100)
		
		self.rune_centerpos = (config.SCREEN_WIDTH / 2, 10)
		self.runes = 0
		self.left_x = config.SCREEN_WIDTH / 2
	
	def getLevel(self, name):
		return self.magics[name].level + 1
	
	def addMp(self, dmp):
		self.mp = min(self.mp + dmp, self.maxmp)		
	
	def addMagic(self, name, mag):
		mag.setController(self)
		self.magics[name] = mag
		self.runes = len(self.magics)
		
		new_x = self.rune_centerpos[0] - self.runes*65/2.0
		d_x = new_x - self.left_x
		self.left_x = new_x
		#print "new_X:", new_x, " dx:", d_x
		
		i = 0
		pos = [new_x, 10]
		runes = sorted(self.magics.iteritems(), key=lambda x: x[0])
		for key, rune in runes:
			rune.setPos(pos)
			pos[0] += 65
		
	def setRecoverTime(self, rt):
		self.recover_time = rt
	
	def update(self, elapsed_time):
	
		# Mana recovery
		if self.mp < self.maxmp:
			self.recoverProgress += elapsed_time
		
		while self.recoverProgress >= self.recover_time and self.mp < self.maxmp:
			self.recoverProgress -= self.recover_time
			self.mp = min(self.mp + 1, self.maxmp)
		
		# mpbar
		self.mpbar.set_amount(self.mp / float(self.maxmp) * 100)
		
		# runes
		for rune in self.magics.itervalues():
			rune.update(elapsed_time)
	
	def draw(self, display):
		self.mpbar.draw(display)
		for rune in self.magics.itervalues():
			rune.draw(display)
		
	def tryCast(self, name):
		mag = self.magics[name]
		manacost = mag.getRequiredMp()
		recast_status = mag.recastStatus()
		
		# different effects can be considered here, like mana cost halving, fastcast, etc..
		
		#print "rcast_status:", recast_status
		
		if (self.mp >= manacost) and (recast_status >= 1.0):
			# Ok, magic can be cast
			mag.increaseUses()
			self.mp -= manacost
			
			#print "succesS with", name, ", uses: ", mag.uses
			return True
		
		else:
			return False