import os

import pygame
from pygame.locals import *

import config
from Sprite import SpriteSequence, SpriteBase

class ImageManager:
	def __init__(self):
		self.images = {}
		
		if not config.__dict__.has_key("image_manager"):
			config.image_manager = self

	def cache(self, imglist):
		for i in imglist:
			self.load(i)
		
	def load(self, image):
		if not self.images.has_key(image):
			self.images[image] = pygame.image.load(os.path.join(config.IMAGE_PATH, image)).convert_alpha()
			
		return self.images[image]

	def __getitem__(self, image):
		if not self.images.has_key(image):
			self.load(image)
			
		return self.images[image]

class SoundManager:
	def __init__(self):
		self.sounds = {}
		
		if not config.__dict__.has_key("sound_manager"):
			config.sound_manager = self

	def cache(self, sndlist):
		for i in sndlist:
			self.load(i)
		
	def load(self, sound):
		if not self.sounds.has_key(sound):
			self.sounds[sound] = pygame.mixer.Sound(os.path.join(config.SOUND_PATH, sound))
			
		return self.sounds[sound]

	def __getitem__(self, sound):
		if not self.sounds.has_key(sound):
			self.load(sound)
			
		return self.sounds[sound]

class FontSizeManager:
	def __init__(self, font_file):
		self.font_file = font_file
		self.sizes = {}

	def __setitem__(self, key, value):
		self.sizes[key] = value

	def __getitem__(self, size):
		if not self.sizes.has_key(size):
			self.sizes[size] = pygame.font.Font(os.path.join(config.FONT_PATH, self.font_file), size)

		return self.sizes[size]

class FontManager:
	def __init__(self):
		self.fonts = {}
		
		if not config.__dict__.has_key("font_manager"):
			config.font_manager = self

	def cache(self, fontlist):
		for i in fontlist:
			self.load(*i)
		
	def load(self, font, size):
		if not self.fonts.has_key(font):
			self.fonts[font] = FontSizeManager()
			
		return self.fonts[font][size]

	def __getitem__(self, font):
		if not self.fonts.has_key(font):
			self.fonts[font] = FontSizeManager(font)
			
		return self.fonts[font]

class SpriteManager:
	def __init__(self, spritelist_file):
		self.bases = {}
		
		f = file(spritelist_file)
		for l in f:
			w = l.split()
			if len(w) == 0 or w[0][0] == "#":
				continue

			sprite_name = w[0]
			file_name = w[1]
			rows = int(w[2])
			columns = int(w[3])
			border = map(lambda x: int(x), w[4:8])
			spacing = map(lambda x: int(x), w[8:10])
			sequence_num = int(w[10])

			sequences = {}

			while sequence_num > 0:
				w = f.next().split()
				if len(w) == 0 or w[0][0] == "#":
					continue

				name = w[0]
				fps = float(w[1])
				if int(w[2]) == 0:
					loop = False
					freeze_frame = int(w[3])
				else:
					loop = True
					freeze_frame = None
				hitbox_num = int(w[4])
				refx = int(w[5])
				refy = int(w[6])
				frame_sequence = map(lambda x: int(x), w[7:])
				
				hitboxes = []
				while hitbox_num > 0:
					w = f.next().split()
					if len(w) == 0 or w[0][0] == "#":
						continue

					hitboxes.append(map(lambda x: int(x), w))
					hitbox_num -= 1

				sequences[ name ] = SpriteSequence(fps, frame_sequence, refx, refy, hitboxes, loop, freeze_frame )
				sequence_num -= 1
			
			#img_surface = image.load(os.path.join(config.SPRITE_PATH, file_name)).convert_alpha()
			img_surface = pygame.image.load(os.path.join(config.SPRITE_PATH, file_name))
			if img_surface.get_bitsize() == 8 and (img_surface.get_flags() & SRCCOLORKEY) == 0:
				img_surface.set_colorkey(255)

			#img_surface = img_surface.convert_alpha()
			img_surface = img_surface.convert()
			#img_surface.set_alpha(128)
			
			max_x = img_surface.get_width() - border[1]
			max_y = img_surface.get_height() - border[3]

			frame_width = (img_surface.get_width() - border[0] - border[1] - (columns - 1) * spacing[0]) / columns
			frame_height = (img_surface.get_height() - border[2] - border[3] - (rows - 1) * spacing[1]) / rows

			frames = []

			y = border[2]
			while y + frame_height <= max_y:
				x = border[0]
				while x + frame_width <= max_x:
					frames.append( pygame.rect.Rect(x, y, frame_width, frame_height) )
					x += frame_width + spacing[0]

				y += frame_height + spacing[1]

			base = SpriteBase(img_surface, frame_width, frame_height, frames)
			self.bases[sprite_name] = base

			for seq_name, seq in sequences.iteritems():
				base.add_anim_sequence(seq_name, seq)

		f.close()

		if not config.__dict__.has_key("sprite_manager"):
			config.sprite_manager = self

	def __len__(self):
		return len(self.sprites)

	def __getitem__(self, name):
		return self.bases[name]

