import random
import pygame
from pygame.locals import *

import config
from Image import Image
from Sprite import *

from cloud import Clouds, Bonuses

import threading
import math
import random

# For mouse gestures
from MotionDetect import MotionDetect, Runes, MouseSampler

from Magic import *

from Enemy import *
from Missile import *
from Area import *

from GameFlow import *

class Scene:
	def __init__(self, display):
		self.display = display

	def run(self):
		pass

class GameScene(Scene):
	def __init__(self, display, gameflow):
		Scene.__init__(self, display)

		self.wizard = Sprite("wizard", "look_left")
		self.wizard.show()

		self.hp_text = TextSprite(10, 10, "Health", (0, 0, 0), 18, "burti_bold.ttf")
		self.mana_text = TextSprite(10, 50, "Mana", (0, 0, 0), 18, "burti_bold.ttf")

		self.interFont = config.font_manager["burti_bold.ttf"][25]
		self.interSprites = []

		self.gameflow = gameflow
		#GameFlow(config.LEVELLIST_FILE)
		#self.act_level_num = -1
		self.end_game = False

		self.balloon_hp = 100
		
		self.hpbar_pos = (20, self.hp_text.height + 10)
		self.manabar_pos = (20, self.mana_text.height + 50)

		self.hpbar = MeterBar(100, 10, self.balloon_hp)
		self.hpbar.set_amount(self.balloon_hp)
		self.hpbar.move(*(self.hpbar_pos))

		self.missile_cast_sound = config.sound_manager["missile_cast.ogg"]
		self.missile_blow_sound = config.sound_manager["missile_blow.ogg"]
		self.magic_whisper = config.sound_manager["madeira.ogg"]
		self.magic_fail = config.sound_manager["zutty.ogg"]
		self.rune_gestures = [config.sound_manager["whisper%d.ogg" % i] for i in (1,2,3,4)]
		
		#self.background = Image("background800x600.png", 0, 0)
		self.background = BackgroundSprite((146, 187, 255), (69, 72, 162))
		
		self.balloon = FuzzySprite("balloon", "still")
		self.balloon.move(config.SCREEN_WIDTH / 2, float(config.SCREEN_HEIGHT) * 0.575 )
		self.balloon.show()

		self.wizard.move(self.balloon.x, self.balloon.y)
		
		self.enemies = []
		self.missiles = []
		self.effects = []
		self.areas = []
		
		# Clouds
		self.clouds = Clouds()
		
		# Bonus package
		self.bonusState = {"multishot":0, "add_hp":False}
		self.bonuses = Bonuses(self.bonusState)
		self.bonuses.setGauss(55000, 10000)
		self.bonuses.setSpeeds([40, 40])
				
		# Particles
		self.particles = []
		
		# Rune detection
		self.runeDetect = Runes()
		
		# MagicController (this is a variaton of a PID controller)
		self.magic = MagicController(self.manabar_pos)
		self.magic.addMagic(
			"triangle", 
			MagicRune(
				"triangle",
				# MP cost at the given level
				[7, 5, 3],
				# Recast time
				[1000, 800, 600],
				# Cast num. to next level
				[20, 40, 60]
			)
		)
		self.magic.addMagic(
			"box", 
			MagicRune(
				"box",
				[10, 7, 5],
				[1500, 1000, 500],
				[20, 40, 60]
			)
		)
		self.magic.addMagic(
			"time", 
			MagicRune(
				"time",
				[20, 15, 10],
				[6500, 3000, 1500],
				[15, 20, 30]
			)
		)
		self.magic.addMagic(
			"lightning", 
			MagicRune(
				"lightning",
				[20, 25, 20],
				[10000, 5000, 3000],
				[10, 20, 30]
			)
		)
		
		# Mouse gestures
		self.motionDetect = MotionDetect()
		
		class MouseDown:
			def __init__(self):
				self.down = False
				self.lock = threading.Lock()
			
			def get(self):
				return self.down
			
			def set(self, val):
				self.lock.acquire()
				self.down = val
				self.lock.release()
		
		# Mouse sprites
		self.nextParticleTime = 0
		self.lastParticlePos = None
		self.particleLimit = 80
		self.particleInterval = 5
		self.particleCutoff = 10
		self.particleStay = 40 # percent
				
		
		self.mouseDown = MouseDown()
		
		self.particleLock = threading.Lock()
		
		self.mouseSampler = MouseSampler(
			pygame.mouse.get_pos, 
			pygame.time.get_ticks,
			pygame.time.wait,
			self.motionDetect.addPoint,
			self.mouseDown,
			self.particleCheck
		)
	
	def bonusCheck(self, elapsed_time):
		if self.bonusState["multishot"]:
			self.bonusState["multishot"] = max(0, self.bonusState["multishot"]-elapsed_time)

		if self.bonusState["add_hp"]:
			self.bonusState["add_hp"] = False
			
			self.balloon_hp += 10
			if self.balloon_hp >= 100:
				self.balloon_hp = 100
			self.hpbar.set_amount(self.balloon_hp)
	
	def particleCheck(self, elapsed_time):
		self.particleLock.acquire()
		self.nextParticleTime -= elapsed_time
		if self.nextParticleTime <= 0 and self.mouseDown.get():
			self.nextParticleTime = self.particleInterval + (self.particleCutoff * max(0, len(self.particles) - self.particleLimit))
			
			x,y = pygame.mouse.get_pos()
			lx, ly = self.lastParticlePos
			
			dx,dy = x-lx, y-ly
			
			if dx or dy:
				dx += random.randint(-1,1)
				dy += random.randint(-1,1)
			
			if random.randint(0,100) < self.particleStay:
				dx = 0
				dy = 0
				
			pps = math.sqrt(dx*dx + dy*dy)
							
			part = MovingSprite("spark", "default")
			part.move(x,y)
			part.set_destination(x+dx, y+dy, pps*10.0)
			
			part.maxlife = 1500 + random.randint(-200, 200) + (max(30-pps,0))*15
			part.life = part.maxlife
			
			part.show()
			part.start()
			
			self.particles.append( part )
			self.lastParticlePos = (x,y)
		self.particleLock.release()

	def run(self):
		self.advance_level()
		
		clock = pygame.time.Clock()

		#pygame.time.set_timer(USEREVENT + 1, 3000)
		
		# Start the mouse sampler in a separate thread
		self.mouseSampler.start()

		self.end_game_reason = None
		
		pygame.event.clear()
		while not self.end_game:
			elapsed_time = clock.tick(config.DESIRED_FPS)

			mpos = pygame.mouse.get_pos()
			for e in self.enemies:
				e.show_info(e.collide_pt(mpos))
			
			#
			# Wear off bonuses
			self.bonusCheck(elapsed_time)			
			
			#
			# Update sprites
			#
			for s in self.interSprites:
				if s.y <= config.SCREEN_HEIGHT * 0.4:
					# no farther then (visula) center
					break
				s.move_rel(0, -elapsed_time / 1000.0 * 50)

			self.magic.update(elapsed_time)
			self.balloon.update(elapsed_time)
			
			self.wizard.move(self.balloon.x, self.balloon.y)
			if mpos[0] < self.wizard.x:
				self.wizard.set_sequence("look_left")
			else:
				self.wizard.set_sequence("look_right")
			
			for what in (self.enemies, self.missiles, self.effects, self.areas):
				for s in what:
					s.update(elapsed_time)
			
			self.clouds.update(elapsed_time)
			self.bonuses.update(elapsed_time)
			
			self.particleLock.acquire()	### LOCK ON
			for s in self.particles: s.update(elapsed_time)
							
			# remove dead particles
			remove_particles = []
			for s in self.particles:
				s.life -= elapsed_time
				if s.life <= 0:
					remove_particles.append( s )
				else:
					s.set_alpha( 255 * float(s.life) / s.maxlife )
			
			for s in remove_particles:
				self.particles.remove(s)
			
			self.particleLock.release() ### LOCK OFF
				
				
			# check for missile collisions
			remove_missiles = []
			for m in self.missiles:
				# check enemy hits
				collide_list = m.collides(self.enemies)
				for enemy, rect in collide_list:
					self.missile_hit(enemy, m, rect)
					if m not in remove_missiles:
						remove_missiles.append(m)
				
				# check bonus hits
				collide_list = m.collides(self.bonuses.clouds)
				for bonus, rect in collide_list:
					self.missile_hit(bonus, m, rect)
					if m not in remove_missiles:
						remove_missiles.append(m)

			for m in remove_missiles:
				self.missiles.remove(m)
			
			# check for area collision
			for area in self.areas:
				# collide only when area is active
				if not area.is_active():
					continue
				
				# check enemy hits
				collide_list = area.collides(self.enemies)
				for enemy, rect in collide_list:
					self.area_hit(enemy, area, rect)
				
				# check bonus hits
				collide_list = area.collides(self.bonuses.clouds)
				for bonus, rect in collide_list:
					self.area_hit(bonus, area, rect)
	
			# check for balloon collision
			collide_list = self.balloon.collides(self.enemies)
			for enemy, rect in collide_list:
				self.hit_balloon(enemy, rect)
			
			#
			# Display animations
			#
			self.background.draw(self.display)
			#self.display.fill((0,0,0))

			self.clouds.draw(self.display)
			self.bonuses.draw(self.display)
			
			self.hp_text.draw(self.display)
			self.mana_text.draw(self.display)
			
			self.hpbar.draw(self.display)
			self.magic.draw(self.display)
			
			self.wizard.draw(self.display)
			self.balloon.draw(self.display)
			for what in (self.enemies, self.missiles, self.effects, self.areas):
				for s in what:
					s.draw(self.display)
			
			
			self.particleLock.acquire()	### LOCK ON
			for s in self.particles: s.draw(self.display)
			self.particleLock.release() ### LOCK OFF
			
			for s in self.interSprites:
				s.draw(self.display)
			
			# Event processing
			for event in pygame.event.get():
				if event.type == KEYDOWN:
					if event.key == K_ESCAPE:
						self.end_game = True
						self.end_game_reason = 1
						#self.mouseSampler.stop()
						#self.mouseSampler.join()
						#return None

					elif event.key == K_F12:
						pygame.image.save(self.display, "screenshot.tga")
						
				elif event.type == MOUSEBUTTONDOWN:
					if event.button == 1:
						# Begin detecting mouse gestures
						if not self.mouseDown.get():
							if random.randint(0,100) < 30:
								self.rune_gestures[random.randint(0,3)].play()
							self.mouseDown.set(True)
							self.lastParticlePos = pygame.mouse.get_pos()
							self.motionDetect.__init__()
					
										
				elif event.type == MOUSEBUTTONUP:
					if event.button == 1 and \
						(isinstance(self.act_stage, WaveStage) or \
						 isinstance(self.act_stage, WaitStage)):
					
						# End detecting mouse gesture
						if self.mouseDown.get() and self.motionDetect:
							for snd in self.rune_gestures:
								snd.fadeout(500)
							self.mouseDown.set(False)
							infoVector = self.motionDetect.returnVector()
							rune = self.runeDetect.whichRune(infoVector)
							if rune:
								# A rune is detected
								# Here we should also check if there is enough mana to cast that
								# rune, and wether that rune is enabled, etc..
							
								rune_name = rune.getName()
								#print rune.getName()
								if rune_name == "triangle":
									if self.magic.tryCast(rune_name):
										# Cast a hacked iceball for now
										pos = rune.getCenter(infoVector)
										self.cast_missile(Iceball, self.magic.getLevel(rune_name), pos, 100.0)
									else:
										# effect: unable to cast
										self.magic_whisper.play()
										

								elif rune_name == "box":
									if self.magic.tryCast(rune_name):
										pos = rune.getCenter(infoVector)
										self.cast_missile(Fireball, self.magic.getLevel(rune_name), pos, 150.0)
									else:
										# effect: unable to cast
										self.magic_whisper.play()
										
								
								elif rune_name == "time":
									if self.magic.tryCast(rune_name):
										pos = rune.getCenter(infoVector)
										self.cast_area(Wind, self.magic.getLevel(rune_name), pos)
										#micsina a self.magic.getLevel(rune_name) szintu szel
									else:
										self.magic_whisper.play()										
								
								elif rune_name == "lightning":
									if self.magic.tryCast(rune_name):
										pos = rune.getCenter(infoVector)
										self.cast_area(Thunder, self.magic.getLevel(rune_name), pos)
										#micsina a self.magic.getLevel(rune_name) szintu villam
									else:
										self.magic_whisper.play()										
										
							else:
								# Rune drawing failed, make some sound..
								self.magic_fail.play()

				elif event.type == USEREVENT + 1:
					# Spawn enemy event
					self.spawn_enemies()
					
				elif event.type == USEREVENT + 2:
					# Wait stage is over
					pygame.time.set_timer(USEREVENT + 2, 0)
					self.advance_stage()

				elif event.type == USEREVENT + 3:
					# Advance intermission
					pygame.time.set_timer(USEREVENT + 3, 0)
					self.advance_intermission()
					
			pygame.display.flip()

		pygame.time.set_timer(USEREVENT + 1, 0)
		pygame.time.set_timer(USEREVENT + 2, 0)
		pygame.time.set_timer(USEREVENT + 3, 0)
		
		self.mouseSampler.stop()
		self.mouseSampler.join()

		if self.end_game_reason == 3:
			game_over = TextSprite(config.SCREEN_WIDTH / 2, config.SCREEN_HEIGHT /2, "Game Over", (255, 255, 255), 40, "burti_bold.ttf", True, (0,0,0), 4)
			game_over.set_alignment(0, 0)
			self.end_game = False

			pygame.time.set_timer(USEREVENT + 1, 10000)
			
			pygame.event.clear()
			
			while not self.end_game:
				elapsed_time = clock.tick(config.DESIRED_FPS)

				self.background.draw(self.display)
				game_over.draw(self.display)
				
				for event in pygame.event.get():
					if event.type == KEYDOWN:
						self.end_game = True

					elif event.type == USEREVENT + 1:
						pygame.time.set_timer(USEREVENT + 1, 0)
						self.end_game = True

				pygame.display.flip()

	def enemy_left_screen(self, enemy):
		self.enemies.remove(enemy)
		self.wave_count += 1
		self.check_wave_stage()

	def enemy_killed(self, enemy):
		self.wave_count += 1
		self.check_wave_stage()

	def check_wave_stage(self):
		if self.wave_count == self.act_stage.enemy_count:
			pygame.time.set_timer(USEREVENT + 1, 0)
			self.advance_stage()

	def cast_area(self, mclass, level, pos):
		area = mclass(level)
		area.move(pos[0], pos[1])
		area.duration_over_callback(self.areas.remove, area)
		area.show()

		self.areas.append(area)

	def cast_missile(self, mclass, level, dest, extra_speed = 0):
		
		dlist = [(0,0)]
		if self.bonusState["multishot"] > 0:
			dlist = []
			for i in xrange(3):
				dlist.append( (random.randint(-40, 40), random.randint(-40, 40)) )
		
		for dx, dy in dlist:
			
			if dest[0] > self.balloon.x:
				missile = mclass(level, False)
				missile.move(self.balloon.x + missile.base.width / 2, self.balloon.y)
			else:
				missile = mclass(level, True)
				missile.move(self.balloon.x - missile.base.width / 2, self.balloon.y)
											
			missile.out_of_screen_callback(self.missiles.remove, missile)
			missile.set_destination(dest[0]+dx, dest[1]+dy, missile.getSpeed() + extra_speed)
			missile.show()
			missile.start()
				
			self.missiles.append(missile)
	
			self.missile_cast_sound.play()

	def area_hit(self, enemy, area, hit_rect):
		enemy.enter_area(area)
		if enemy.hp <= 0:	
			self.missile_blow_sound.play()
			
			blow = Sprite("blow", "fireball")
			blow.anim_end_callback(self.effects.remove, blow)
			blow.move(hit_rect.left + hit_rect.width / 2, hit_rect.top + hit_rect.height / 2)
			blow.show()
			self.effects.append(blow)

			self.magic.addMp(1)
			self.enemy_killed(enemy)
			self.enemies.remove(enemy)

	def missile_hit(self, enemy, missile, hit_rect):
		self.missile_blow_sound.play()
		
		mtype = missile.kind
		etype = enemy.kind
		
		if missile.to_left:
			blow = Sprite(mtype.blow_sprite, mtype.blow_left)
		else:
			blow = Sprite(mtype.blow_sprite, mtype.blow_right)
						
		blow.anim_end_callback(self.effects.remove, blow)
		blow.move(hit_rect.left + hit_rect.width / 2, hit_rect.top + hit_rect.height / 2)
		blow.show()
		self.effects.append(blow)
					
		missile.stop()
		missile.show(False)

		enemy.missile_hit(missile)
		if enemy.hp <= 0:	
			self.magic.addMp(1)
			self.enemy_killed(enemy)
			self.enemies.remove(enemy)

	def advance_intermission(self):
		if self.intermission_effect is not None:
			self.effects.remove(self.intermission_effect)
			
		self.act_intermission_stage += 1
		if self.act_intermission_stage < len(self.act_stage.sub_stages):
			stage_data = self.act_stage.sub_stages[self.act_intermission_stage]

			if isinstance(stage_data, IntermissionTextinfo):
				#self.intermission_effect = Sprite("spark", "default")
				#self.intermission_effect.move(10, 400)
				#self.intermission_effect.show()
			
				#self.effects.append(self.intermission_effect)
				maxw = 600
				x = config.SCREEN_WIDTH / 2
				color = (255,255,255)
				
				lasty = config.SCREEN_HEIGHT
				
				for line in stage_data.text_lines:
					words = line.split()
					w,h = 0,0
					print "WS", words
					while len(words):
						text = ""
						w, h = self.interFont.size(text + " " +words[0])
						while len(words) and (w <= maxw):
							word = words.pop(0)
							text += " " + word
							if len(words):
								w, h = self.interFont.size(text + " " +words[0])
							
						txtSprite = TextSprite(x, lasty, text, color, 25, "burti_bold.ttf", True, (0, 0, 0))
						txtSprite.set_alignment(0, -1)
						self.interSprites.append(txtSprite)
						lasty += h+5
			
				pygame.time.set_timer(USEREVENT + 3, stage_data.time * 1000)
		else:
			self.interSprites = []
			self.advance_stage()
			
	def advance_level(self):
		self.act_level = self.gameflow.next()
		#self.act_level_num += 1
		#if self.act_level_num == len(self.gameflow.sub_stages):
		if self.act_level is None:
			self.end_game = True
			self.end_game_reason = 2
			return False
		
		#self.act_level = self.gameflow.sub_stages[self.act_level_num]

		#self.act_stage_num = -1
		return self.advance_stage()

	def advance_stage(self):
		#self.act_stage_num += 1
		self.act_stage = self.act_level.next()
		#if self.act_stage_num == len(self.act_level.sub_stages):
		if self.act_stage is None:
			if not self.advance_level():
				return False
		#else:
		#	self.act_stage = self.act_level.sub_stages[self.act_stage_num]
		
		if isinstance(self.act_stage, WaitStage):
			pygame.time.set_timer(USEREVENT + 2, self.act_stage.wait_secs * 1000)
					
		elif isinstance(self.act_stage, WaveStage):
			self.avail_enemy_classes = []
			
			self.max_enemy_freq = 0
			for wave_enemy_info in self.act_stage.enemy_types:
				self.max_enemy_freq += wave_enemy_info.freq
				self.avail_enemy_classes.append( (enemy_classes[wave_enemy_info.enemy_type], self.max_enemy_freq) )
				
			self.wave_count = 0
			self.spawn_count = 0
					
			# Start spawn timer
			pygame.time.set_timer(USEREVENT + 1, self.act_stage.time_delay * 1000)

		elif isinstance(self.act_stage, IntermissionStage):
			self.intermission_effect = None
			self.act_intermission_stage = -1
			self.advance_intermission()

		elif isinstance(self.act_stage, HintStage):
			hint = HintTextSprite(config.SCREEN_WIDTH / 2, config.SCREEN_HEIGHT - 50, self.act_stage.hint_text, (255, 255, 255), 24, "burti_bold.ttf", self.act_stage.fade_in_time, self.act_stage.display_time, self.act_stage.fade_out_time)
			hint.disappear_callback(self.hint_disappeared, hint)
			self.effects.append(hint)
					
		print "Advanced to the next stage"
		print self.act_stage

		return True

	def hint_disappeared(self, hint):
		self.effects.remove(hint)
		self.advance_stage()

	def hit_balloon(self, enemy, hit_rect):
		self.missile_blow_sound.play()
		blow = Sprite("blow", "fireball")
		blow.anim_end_callback(self.effects.remove, blow)
		blow.move(hit_rect.left + hit_rect.width / 2, hit_rect.top + hit_rect.height / 2)
		blow.show()

		self.effects.append(blow)
					
		enemy.show(False)
		self.enemies.remove(enemy)
		
		self.wave_count += 1
		self.check_wave_stage()

		self.balloon_hp -= 10
		if self.balloon_hp < 0:
			self.balloon_hp = 0
			self.end_game = True
			self.end_game_reason = 3
			
		self.hpbar.set_amount(self.balloon_hp)

	def spawn_enemies(self):
		if not isinstance(self.act_stage, WaveStage):
			return

		for i in xrange(self.act_stage.group_size):
			if self.spawn_count == self.act_stage.enemy_count:
				break

			n = random.randint(0, self.max_enemy_freq)
			j = 0
			while j < len(self.avail_enemy_classes) and n > self.avail_enemy_classes[j][1]:
				j += 1
			
			enemy_class, freq = self.avail_enemy_classes[j]
			
			enemy = enemy_class(1, random.choice((True, False)), random.randint(30 , config.SCREEN_HEIGHT - 30))
			enemy.out_of_screen_callback(self.enemy_left_screen, enemy)
			enemy.show()

			self.enemies.append(enemy)

			self.spawn_count += 1

class MainMenuScene(Scene):
	def __init__(self, display):
		self.display = display

		self.background = BackgroundSprite((146, 187, 255), (69, 72, 162))

		inactive_color = (255, 255, 255)
		active_color = (255, 255, 0)
		menupt_dist = 70

		self.game_title = TextSprite(config.SCREEN_WIDTH / 2, 20, "Sky Journey", (255, 255, 255), 60, "burti_bold.ttf", True, (0, 0, 0), 4)
		self.game_title.set_alignment(0, -1)
		
		menu_pts = ["Story Mode - Easy", "Story Mode - Hard", "Endless Mode", "How to play", "Quit"]

		y = (config.SCREEN_HEIGHT - menupt_dist * (len(menu_pts) - 1)) / 2
		self.menu = []
		for menu_text in menu_pts:
			menu_item = [
				TextSprite(config.SCREEN_WIDTH / 2, y, menu_text, inactive_color, 30, "burti_bold.ttf", True, (0, 0, 0), 2),
				TextSprite(config.SCREEN_WIDTH / 2, y, menu_text, active_color, 30, "burti_bold.ttf", True, (0, 0, 0), 6),
			]
			menu_item[0].set_alignment(0, -1)
			menu_item[1].set_alignment(0, -1)
			self.menu.append(menu_item)

			y += 70

		self.enemy_classes = []
		for eclass in enemy_classes.itervalues():
			self.enemy_classes.append(eclass)

		self.effects = []
		self.clouds = Clouds()

	def run(self):
		clock = pygame.time.Clock()

		act_menu = 0
		
		pygame.time.set_timer(USEREVENT + 1, random.randint(4000, 8000))

		quit = False

		pygame.event.clear()
		while not quit:
			elapsed_time = clock.tick(config.DESIRED_FPS)
			
			self.clouds.update(elapsed_time)
			for e in self.effects:
				e.update(elapsed_time)

			self.background.draw(self.display)
			self.clouds.draw(self.display)

			for e in self.effects:
				e.draw(self.display)

			self.game_title.draw(self.display)
			
			for i in xrange(len(self.menu)):
				if i == act_menu:
					self.menu[i][1].draw(self.display)
				else:
					self.menu[i][0].draw(self.display)
			
			for event in pygame.event.get():
				if event.type == KEYDOWN:
					if event.key == K_ESCAPE:
						quit = True
						act_menu = -1

					elif event.key == K_RETURN:
						quit = True

					elif event.key == K_UP:
						if act_menu == 0:
							act_menu = len(self.menu) - 1
						else:
							act_menu -= 1

					elif event.key == K_DOWN:
						if act_menu == len(self.menu) - 1:
							act_menu = 0
						else:
							act_menu += 1

				elif event.type == USEREVENT + 1:
					self.spawn_enemy()
					pygame.time.set_timer(USEREVENT + 1, random.randint(4000, 8000))

			pygame.display.flip()
		
		pygame.time.set_timer(USEREVENT + 1, 0)

		return act_menu
		

	def spawn_enemy(self):
		enemy_class = random.choice(self.enemy_classes)
		
		enemy = enemy_class(1, random.choice((True, False)), random.randint(30 , config.SCREEN_HEIGHT - 30))
		enemy.show()
		enemy.out_of_screen_callback(self.effects.remove, enemy)

		self.effects.append(enemy)

class LoadingScene(Scene):
	def __init__(self, display):
		self.display = display
		
		self.background = BackgroundSprite((146, 187, 255), (69, 72, 162))
		self.loading_text = TextSprite(config.SCREEN_WIDTH / 2, config.SCREEN_HEIGHT / 2, "Loading please wait...", (255, 255, 255), 40, "burti_bold.ttf", True, (0, 0, 0), 4)
		self.loading_text.set_alignment(0, 0)

	def run(self):
		class CacheResources(threading.Thread):
			def __init__(self):
				threading.Thread.__init__(self)

			def run(self):
				if config.enable_music:
					config.sound_manager.cache(("141th_street_blues.ogg",))
		
		clock = pygame.time.Clock()

		cache_thread = CacheResources()
		cache_thread.start()

		quit = False

		pygame.event.clear()
		while cache_thread.isAlive():
			elapsed_time = clock.tick(1)
			
			self.background.draw(self.display)
			self.loading_text.draw(self.display)

			pygame.display.flip()

		if cache_thread.isAlive():
			cache_thread.stop()
			cache_thread.join()

class HowtoScene(Scene):
	def __init__(self, display):
		self.display = display
		
		self.background = BackgroundSprite((146, 187, 255), (69, 72, 162))
		howto_text = """
			Welcome to Sky Journey! In the following you'll be given some probably helpful advices.
			In this game you have to use your mouse to draw different kinds of runes into the air
			like a real wizard. This is not an easy task although. You have to be very precise while
			drawing. The first important thing that the starting point and direction MATTERS! It's
			obvious for a real wizard. You have to draw the runes starting from the red end and moving
			toward the yellow end. Simple isn't it? ... You're right, it isn't! Because there are
			other aspects. You have to be very careful to draw straight segments relatively quickly
			and stop on the corners just for a flash. One more thing: apprentice wizards tend to draw
			tall triangles. That's a bad habit because this usually leads the spell to fail.
			"""

		lines = howto_text.splitlines()

		text_spacing = 40
		
		y = (config.SCREEN_HEIGHT - text_spacing * (len(lines) - 1)) / 2
		self.lines = []
		for l in lines:
			l = l.strip()
			if len(l) == 0:
				continue
			text_sprite = TextSprite(config.SCREEN_WIDTH / 2, y, l, (255, 255, 255), 24, "burti_bold.ttf", True, (0, 0, 0), 2)
			text_sprite.set_alignment(0, -1)
			self.lines.append(text_sprite)

			y += text_spacing

	def run(self):
		clock = pygame.time.Clock()

		quit = False

		pygame.event.clear()
		while not quit:
			elapsed_time = clock.tick(1)
			
			self.background.draw(self.display)
			for l in self.lines:
				l.draw(self.display)
			
			for event in pygame.event.get():
				if event.type == KEYDOWN:
					if event.key == K_ESCAPE:
						quit = True

			pygame.display.flip()

